program extractor;

uses
  SysUtils
, StreamEx
, RegExpr
;

const
  cUnicodeVersion = '14.0';
  cEmojiList = 'emoji-list_%s.html';
  cEmojiFullList = 'full-emoji-list_%s.html';

  cRegExpBigHead =
    '<tr><th colspan=''\d+'' class=''bighead''><a href=''#\S+'' name=''\S+''>([\S\s]+)<\/a><\/th><\/tr>';
  cRegExpMediumHead =
    '<tr><th colspan=''\d+'' class=''mediumhead''><a href=''#\S+'' name=''\S+''>([\S\s]+)<\/a><\/th><\/tr>';
  cRegExpCodePoint =
    '<td class=''code''><a href=''#[0-9a-f_]+'' name=''[0-9a-f_]+''>U\+([0-9A-F]+)[\S\s]*<\/a><\/td>';
  cRegExpName =
    '<td class=''name''>([\S\s]+)</td>';

var
  InputFile: TFileReader;
  line: String;
  regexp: TRegExpr;

begin
  // Emoji List
  InputFile:= TFileReader.Create(Format('../data/' + cEmojiList, [cUnicodeVersion]));
  try
    repeat
      line:= InputFile.ReadLine;
      // Category
      if Pos('bighead', line) > 0 then
      begin
        //WriteLN('Line: ', line);
        regexp:= TRegExpr.Create(cRegExpBigHead);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('  ',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
      // Sub-Category
      if Pos('mediumhead', line) > 0 then
      begin
        //WriteLN(line);
        regexp:= TRegExpr.Create(cRegExpMediumHead);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('    ',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
      // Code Point
      if Pos('class=''code''', line) > 0 then
      begin
        //WriteLN('      ', line);
        regexp:= TRegExpr.Create(cRegExpCodePoint);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('      U+',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
      // Name
      if Pos('class=''name''', line) > 0 then
      begin
        //WriteLN('        ',line);
        regexp:= TRegExpr.Create(cRegExpName);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('        ',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
    until InputFile.Eof;
  finally
    InputFile.Close;
    InputFile.Free;
  end;

  WriteLN;

  // Full Emoji List
  {InputFile:= TFileReader.Create(Format('../data/' + cEmojiFullList, [cUnicodeVersion]));
  try
    repeat
      line:= InputFile.ReadLine;
      if Pos('bighead', line) > 0 then
      begin
        //WriteLN('Line: ', line);
        regexp:= TRegExpr.Create(cRegExpBigHead);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('  ',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
      if Pos('mediumhead', line) > 0 then
      begin
        //WriteLN(line);
        regexp:= TRegExpr.Create(cRegExpMediumHead);
        try
          if regexp.Exec(line) then
          begin
            WriteLN('    ',regexp.Match[1]);
          end;
        finally
          regexp.Free;
        end;
      end;
    until InputFile.Eof;
  finally
    InputFile.Close;
    InputFile.Free;
  end;}

  WriteLN;
end.

